/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */ 
#include <vdkodbc/vdkodbc.h>

/*
 */
bool
VDKODBCObject::SQL_OK(SQLINTEGER rslt)
{
	return ((rslt == SQL_SUCCESS) ||
		(rslt == SQL_SUCCESS_WITH_INFO));
}

/*
 */
bool
VDKODBCObject::SQLTest(SQLINTEGER rslt, SQLHENV henv, SQLHDBC hconn, SQLHSTMT hstmt)
{
	SQLCHAR		SQLstat[10];	// Status SQL
	SQLCHAR		SQLmsg[VDKODBC_MAX_ERR_BUF];	// Message buffer
	SQLSMALLINT	SQLmlen;
	
	if (!SQL_OK(rslt)) {
		LastNumError = rslt;
		SQLError(henv, hconn, hstmt, SQLstat, &rslt, SQLmsg, VDKODBC_MAX_ERR_BUF, &SQLmlen);
		printf("%s\n", (const char*)SQLmsg);
		LastError = (const char*)SQLmsg;
		return false;
	}
	return true;
}

/*
 */
void
VDKODBCObject::Err(SQLINTEGER errnum, const char *format, ...)
{
	va_list argptr;
	int cnt;
	VDKString TxtErr;

	// alloc a large string
	char* szBuffer = new char[MAXPRINTFLEN];
	// Memory error, nothing is done yet
	if (szBuffer == NULL) {
		LastNumError = VDKODBC_ERR_MEMORY;
		LastError = "[VDKODBC Error] Memory error";
		return;
	}
	// constructing the new string
	va_start(argptr, format);
	cnt = g_vsnprintf(szBuffer, MAXPRINTFLEN, format, argptr);
	va_end(argptr);
	if (cnt == EOF) {
		delete[] szBuffer;
		LastNumError = VDKODBC_ERR_EOF_REACHED;
		LastError = "[VDKODBC Error] EOF reached";
		return;
	}
	switch (errnum) {
		case VDKODBC_ERR_COLUMN_NOT_FOUND:
			TxtErr = "Column not found";
			break;
		case VDKODBC_ERR_TABLE_NOT_FOUND:
			TxtErr = "Table not found";
			break;
		case VDKODBC_ERR_CONN_NEEDED:
			TxtErr = "Connection needed for this operation";
			break;
		case VDKODBC_ERR_INVALID_CONN_POINTER:
			TxtErr = "Invalid connection pointer";
			break;
		case VDKODBC_ERR_INVALID_COLUMN_NAME:
			TxtErr = "Invalid column name";
			break;
		case VDKODBC_ERR_INVALID_COLUMN_NUMBER:
			TxtErr = "Invalid column number";
			break;
		case VDKODBC_ERR_NULL_FIELD_LENGTH:
			TxtErr = "Field length is zero";
			break;
		case VDKODBC_ERR_MEMORY:
			TxtErr = "Insufficient memory";
			break;
		default:
			TxtErr = "Unkown error";
	}
	LastNumError = errnum;
	LastError.Sprintf("[VDKODBC Error] %d : %s, %s", LastNumError, TxtErr.c_str(), szBuffer);
	delete[] szBuffer;
}


