/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */ 
#include <vdkodbc/vdkodbctable.h>

/*
 */
void
VDKODBCTable::SetStrValue(int index, const char *value)
{
	switch (index) {
		case VDKODBC_TABLE_CAT:
			Catalog = value;
			break;
		case VDKODBC_TABLE_SCH:
			Schema = value;
			break;
		default:
			VDKODBCBase::SetStrValue(index, value);
	}
}

/*
 */
int
VDKODBCTable::LoadColumnsAttr(SQLHENV henv, SQLHDBC hconn)
{
	SQLHSTMT	Hstmt;
	SQLRETURN	rslt;
	SQLCHAR		buffer[VDKODBC_MAXCOLNAMELEN] = "";
	SQLINTEGER	len;
	SQLSMALLINT typedata = 0, sqlsint, nbcols;
	SQLINTEGER	sqlint, sqlint1;
	int			index = 0;
	
	// Delete all table descriptors
	ColumnListIterator it(Columns);
	for (; it; it++) delete it.current();
	Columns.flush();
	// Allocate a statement handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_STMT, hconn, &Hstmt), henv, hconn, Hstmt))
		return LastNumError;
/*	Not supported yet by the driver manager plm 20/01/2001
	// Set SQL_ATTR_METADATA_ID to false
	if (!SQLTest(SQLSetStmtAttr(Hstmt, SQL_ATTR_METADATA_ID, (void*)SQL_FALSE, 0), henv, hconn, Hstmt) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
*/
	// Try getting table column names
	if (!SQLTest(SQLColumns(Hstmt, (SQLCHAR*)GetCatalog().c_str(), GetCatalog().size(),
							(SQLCHAR*)GetSchema().c_str(), GetSchema().size(),
							(SQLCHAR*)Name.c_str(), Name.size(), NULL, 0),
							henv, hconn, Hstmt)) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
	if (!SQLTest(SQLNumResultCols(Hstmt, &nbcols))) return LastNumError;
	// Fetch the data
	rslt = SQLFetch(Hstmt);  
	while ((rslt != SQL_NO_DATA) && SQL_OK(rslt)) {
		// Create a VDKODBCColumn object
		VDKODBCColumn *column = new VDKODBCColumn();
		if (!column) {
			Err(VDKODBC_ERR_MEMORY, "VDKODBCTable::LoadColumnsAttr, Table name : %s", Name.c_str());
			SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
			return VDKODBC_ERR_MEMORY;
		}
		//	For ODBC 3.0 level
		for (int i = VDKODBC_SQLCOLUMNS_NAME; i <= nbcols; i++) {
			switch (i) {
				case VDKODBC_SQLCOLUMNS_NAME:
					index = VDKODBC_NAME;
					typedata = SQL_C_CHAR;
					break;
				case VDKODBC_SQLCOLUMNS_DATA_TYPE:
					index = VDKODBC_COL_TYPE;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_TYPE:
					index = VDKODBC_TYPENAME;
					typedata = SQL_C_CHAR;
					break;
				case VDKODBC_SQLCOLUMNS_SIZE:
					index = VDKODBC_COL_SIZE;
					typedata = SQL_C_SLONG;
					break;
				case VDKODBC_SQLCOLUMNS_BUFLENGTH:
					index = VDKODBC_COL_BUFLEN;
					typedata = SQL_C_SLONG;
					break;
				case VDKODBC_SQLCOLUMNS_DECDIGITS:
					index = VDKODBC_COL_DECDIGITS;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_PRECISION:
					index = VDKODBC_COL_RADIX;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_NULLABLE:
					index = VDKODBC_COL_NULLABLE;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_REMARKS:
					index = VDKODBC_REMARKS;
					typedata = SQL_C_CHAR;
					break;
				// ODBC 3.0 begin here
				case VDKODBC_SQLCOLUMNS_DEFAULT:
					index = VDKODBC_COL_DEF;
					typedata = SQL_C_CHAR;
					break;
				case VDKODBC_SQLCOLUMNS_SQL_DATA_TYPE:
					index = VDKODBC_COL_SQLTYPE;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_DATETIME_SUB:
					index = VDKODBC_COL_SQLDTSUB;
					typedata = SQL_C_SSHORT;
					break;
				case VDKODBC_SQLCOLUMNS_BYTE_LENGTH:
					index = VDKODBC_COL_BYTELEN;
					typedata = SQL_C_SLONG;
					break;
				case VDKODBC_SQLCOLUMNS_ORDPOS:
					index = VDKODBC_COL_ORDPOS;
					typedata = SQL_C_SLONG;
					break;
				case VDKODBC_SQLCOLUMNS_ISNULLABLE:
					index = VDKODBC_COL_ISNULLABLE;
					typedata = SQL_C_CHAR;
					break;
				default:
					index = 0;
					typedata = 0;
					SQLGetData(Hstmt, i, SQL_C_CHAR, buffer, VDKODBC_MAXCOLNAMELEN, &len);
					//printf("default : %s\n", buffer);
					buffer[0] = 0;
			}
			switch(typedata) {
				case SQL_C_CHAR:
					// Postgres unixODBC driver does not answer properly here - plm 2003/02/15
					if (SQLTest(SQLGetData(Hstmt, i, SQL_C_CHAR, buffer, VDKODBC_MAXCOLNAMELEN, &len), henv, hconn, Hstmt)) {
						column->SetStrValue(index, (const char*)buffer);
						//printf("SQL_C_CHAR\t\ti = %d, index = %d : %s\n", i, index, buffer);
						buffer[0] = 0;
					}
					break;
				case SQL_C_SSHORT:
					if (SQLTest(SQLGetData(Hstmt, i, SQL_C_SSHORT, &sqlsint, 0, &sqlint), henv, hconn, Hstmt)) {
						column->SetIntValue(index, sqlsint);
						//printf("SQL_C_SSHORT\ti = %d, index = %d : %d\n", i, index, sqlsint);
						sqlsint = 0;
					}
					break;
				case SQL_C_SLONG:
					if (SQLTest(SQLGetData(Hstmt, i, SQL_C_SLONG, &sqlint, 0, &sqlint1), henv, hconn, Hstmt)) {
						column->SetIntValue(index, sqlint);
						//printf("SQL_C_SLONG\t\ti = %d, index = %d : %ld\n", i, index, sqlint);
						sqlint = 0;
					}
					break;
			}
		}
		// Add the pointer to the list
		Columns.add(column);
		rslt = SQLFetch(Hstmt);  
	}
	// Just to return a single possible value instead of two
	if (SQL_OK(rslt) || rslt == SQL_NO_DATA) rslt = SQL_SUCCESS;
	else SQLTest(rslt, henv, hconn, Hstmt);
	// Free statement handle
	SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
	return rslt;
}

/*
 */
unsigned int
VDKODBCTable::GetNbColumns()
{
	return Columns.size();
}

/*
 */
VDKODBCColumn*
VDKODBCTable::GetPColumn(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return NULL;
	return Columns[colnum];
}

/*
 */
VDKODBCColumn*
VDKODBCTable::GetPColumn(VDKString colname)
{
	ColumnListIterator it(Columns);
	for (; it; it++) {
		if (it.current()->GetName() == colname)
			return it.current();
	}
	Err(VDKODBC_ERR_COLUMN_NOT_FOUND,
		"VDKODBCConnection::GetPColumn, column name : %s",
		colname.c_str());
	return NULL;
}

/*
 */
VDKString
VDKODBCTable::GetColumnName(unsigned int colnum)
{
	VDKString Result;

	if (colnum >= GetNbColumns()) return Result;
	Result = Columns[colnum]->GetName();
	return Result;
}

/*
 */
int
VDKODBCTable::GetColumnNum(VDKString colname)
{
	int rslt = 0;
	ColumnListIterator it(Columns);
	for (; it; it++) {
		if (it.current()->GetName() == colname)
			return rslt;
		rslt++;
	}
	Err(VDKODBC_ERR_COLUMN_NOT_FOUND,
		"VDKODBCConnection::GetColumnNum, column name : %s",
		colname.c_str());
	return VDKODBC_ERR_COLUMN_NOT_FOUND;
}

/*
 */
VDKString
VDKODBCTable::GetColumnTypeName(unsigned int colnum)
{
	VDKString Result;

	if (colnum >= GetNbColumns()) return Result;
	Result = Columns[colnum]->GetTypeName();
	return Result;
}

/*
 */
VDKString
VDKODBCTable::GetColumnTypeName(VDKString colname)
{
	VDKString Result;
	
	int colnum = GetColumnNum(colname);
	if (colnum == VDKODBC_ERR_COLUMN_NOT_FOUND) return Result;
	return GetColumnTypeName(colnum);
}

/*
 */
VDKString
VDKODBCTable::GetColumnRemarks(unsigned int colnum)
{
	VDKString Result;

	if (colnum >= GetNbColumns()) return Result;
	Result = Columns[colnum]->GetRemarks();
	return Result;
}

/*
 */
VDKString
VDKODBCTable::GetColumnRemarks(VDKString colname)
{
	VDKString Result;
	
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return Result;
	return GetColumnRemarks(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnSize(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return Columns[colnum]->GetSize();
}

/*
 */
int
VDKODBCTable::GetColumnSize(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnSize(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnBufLength(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	// ODBC 3.0
	int rslt = Columns[colnum]->GetByteLength();
	if (rslt != 0) return rslt;
	// ODBC 2.0
	return Columns[colnum]->GetBufLength();
}

/*
 */
int
VDKODBCTable::GetColumnBufLength(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnBufLength(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnDecDigits(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return Columns[colnum]->GetDecDigits();
}

/*
 */
int
VDKODBCTable::GetColumnDecDigits(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnDecDigits(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnRadix(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return Columns[colnum]->GetRadix();
}

/*
 */
int
VDKODBCTable::GetColumnRadix(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnRadix(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnNullable(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	VDKString temp = Columns[colnum]->GetIsNullable();
	if (temp.isNull()) {
		if (Columns[colnum]->GetNullable() == SQL_NULLABLE) return VDKODBC_NULLABLE;
		else return VDKODBC_NOT_NULLABLE;
	}
	else {
		if (temp == VDKString("NO")) return VDKODBC_NOT_NULLABLE;
		else return VDKODBC_NULLABLE;
	}
}

/*
 */
int
VDKODBCTable::GetColumnNullable(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnNullable(colnum);
}

/*
 */
VDKString
VDKODBCTable::GetColumnIsNullable(unsigned int colnum)
{
	VDKString nullable;
	
	if (colnum >= GetNbColumns()) return nullable;
	nullable = Columns[colnum]->GetIsNullable();
	if (nullable.isNull()) {
		if (Columns[colnum]->GetNullable() == SQL_NULLABLE) nullable = "YES";
		else nullable = "NO";
	}
	return nullable;
}

/*
 */
VDKString
VDKODBCTable::GetColumnIsNullable(VDKString colname)
{
	VDKString nullable;
	
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return nullable;
	return GetColumnIsNullable(colnum);
}

/*
 */
VDKString
VDKODBCTable::GetColumnDefaultValue(unsigned int colnum)
{
	VDKString Result;

	if (colnum >= GetNbColumns()) return Result;
	Result = Columns[colnum]->GetDefault();
	return Result;
}

/*
 */
VDKString
VDKODBCTable::GetColumnDefaultValue(VDKString colname)
{
	VDKString Result;

	int colnum = GetColumnNum(colname);
	if (colnum < 0) return Result;
	return GetColumnDefaultValue(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnDateTimeSub(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return Columns[colnum]->GetDateTimeSub();
}

/*
 */
int
VDKODBCTable::GetColumnDateTimeSub(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnDateTimeSub(colnum);
}

/*
 */
int
VDKODBCTable::GetColumnOrdinalPos(unsigned int colnum)
{
	if (colnum >= GetNbColumns()) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return Columns[colnum]->GetOrdinalPos();
}

/*
 */
int
VDKODBCTable::GetColumnOrdinalPos(VDKString colname)
{
	int colnum = GetColumnNum(colname);
	if (colnum < 0) return VDKODBC_ERR_COLUMN_NOT_FOUND;
	return GetColumnOrdinalPos(colnum);
}

/*
 */
int
VDKODBCTable::LoadPrivileges(SQLHENV henv, SQLHDBC hconn)
{
	//
	// This function is NOT supported by the Postgres driver yet 2001/01/29 plm
	//
	SQLHSTMT	Hstmt;
	SQLRETURN	rslt;
	SQLSMALLINT	nbcols;
	SQLCHAR		buffer[VDKODBC_MAXCOLNAMELEN] = "";
	SQLINTEGER	len;
	int			index = 0;
	
	// Allocate a statement handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_STMT, hconn, &Hstmt), henv, hconn, Hstmt))
		return LastNumError;
/*	Not supported yet by the driver manager plm 20/01/2001
	// Set SQL_ATTR_METADATA_ID to false
	if (!SQLTest(SQLSetStmtAttr(Hstmt, SQL_ATTR_METADATA_ID, (void*)SQL_FALSE, 0), henv, hconn, Hstmt) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
*/
	// Try getting table privileges
	if (!SQLTest(SQLTablePrivileges(Hstmt, (SQLCHAR*)GetCatalog().c_str(), GetCatalog().size(),
									(SQLCHAR*)GetSchema().c_str(), GetSchema().size(),
									(SQLCHAR*)Name.c_str(), Name.size()), henv, hconn, Hstmt)) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
	if (!SQLTest(SQLNumResultCols(Hstmt, &nbcols))) return LastNumError;
	// Fetch the data
	rslt = SQLFetch(Hstmt);  
	while ((rslt != SQL_NO_DATA) && SQL_OK(rslt)) {
		// Create a VDKODBCColumn object
		VDKODBCPriv *priv = new VDKODBCPriv();
		if (!priv) {
			Err(VDKODBC_ERR_MEMORY, "VDKODBCTable::LoadPrivileges, Table name : %s", Name.c_str());
			SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
			return VDKODBC_ERR_MEMORY;
		}
		for (int i = VDKODBC_SQLTABLEPRIV_GRANTOR; i <= nbcols; i++) {
			switch (i) {
				case VDKODBC_SQLTABLEPRIV_GRANTOR:
					index = VDKODBC_PRIV_GRANTOR;
					break;
				case VDKODBC_SQLTABLEPRIV_GRANTEE:
					index = VDKODBC_PRIV_GRANTEE;
					break;
				case VDKODBC_SQLTABLEPRIV_PRIV:
					index = VDKODBC_PRIV_PRIV;
					break;
				case VDKODBC_SQLTABLEPRIV_GRANTABLE:
					index = VDKODBC_PRIV_GRANTABLE;
					break;
			}
			if (SQLTest(SQLGetData(Hstmt, i, SQL_C_CHAR, buffer, VDKODBC_MAXCOLNAMELEN, &len), henv, hconn, Hstmt)) {
				priv->SetValue(index, (const char*)buffer);
				//printf("%d : %s\n", i, buffer);
				buffer[0] = 0;
			}
		}
		// Add the pointer to the list
		Privileges.add(priv);
		rslt = SQLFetch(Hstmt);  
	}
	// Just to return a single possible value instead of two
	if (SQL_OK(rslt) || rslt == SQL_NO_DATA) rslt = SQL_SUCCESS;
	else SQLTest(rslt, henv, hconn, Hstmt);
	// Free statement handle
	SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
	return rslt;
}




