/* testodbc.cc

  testing VDKODBC
*/
#include <stdlib.h>
#include <stdio.h>

#include <vdkodbc/vdkodbc_support.h>

#define DATABASENAME  "myfamily"
#define USERNAME      "postgres"
#define PASSWORD      "postgres"


void GetODBCDataSources(VDKODBCConnection *connect)
{
    VDKString sources;

    // Get ODBC data sources
	if (connect->SQL_OK(connect->GetDataSources(sources)))
		printf("Sources : \n---------\n%s\n", sources.c_str());
	else printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
}

void GetODBCDrivers(VDKODBCConnection *connect)
{
    VDKString drivers;

    // Get ODBC drivers
	if (connect->SQL_OK(connect->GetDrivers(drivers)))
		printf("Drivers : \n---------\n%s\n", drivers.c_str());
	else printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
}

bool ConnectedToDataSource(VDKODBCConnection *connect)
{
    VDKString DataBaseName = DATABASENAME;
    VDKString UserName = USERNAME;
    VDKString PassWord = PASSWORD;
    // Connect to data source
	if (connect->SQL_OK(connect->Connect(DataBaseName, UserName, PassWord))) {
	    printf("Connected to Database %s as user %s\n\n", DataBaseName.c_str(), UserName.c_str());
		return true;
	}
	printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
	printf("\nDatabase '%s' not found...\n\n", DataBaseName.c_str());
	return false;
}

void ShowTablesAttr(VDKODBCConnection *connect)
{
	VDKString temp, ltemp, tables, tablename, type, catalog, schema, columns, remarks, columnname;
	unsigned int j = 0, k = 0, nbcols;

	// Get all table names, attributes and privileges
	if (connect->SQL_OK(connect->LoadTablesAttr()))
		printf("Tables attributes loaded, table quantity : %d\n", connect->GetNbTables());
	else printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
	// Get data sources table name
	for (j = 0; j < connect->GetNbTables(); j++) {
		tablename = connect->GetTableName(j);
		type = connect->GetTableType(j);
		catalog = connect->GetTableCatalog(j);
		schema = connect->GetTableSchema(j);
		remarks = connect->GetTableRemarks(j);
		printf("\nTable %d : %s, type : %s, catalog : %s, schema : %s, remarks : %s\n", j,
				tablename.c_str(), type.c_str(), catalog.c_str(), schema.c_str(), remarks.c_str());
		/*	NOT supported by the Postgres driver 2003/01/18 plm
		// Load table privileges
		if (connect.SQL_OK(connect.LoadTablePrivileges(tablename))) {
			printf("\tTable privileges loaded\n");
		}
		else printf("Error n %d : %s\n", connect.GetLastNumError(), connect.GetLastError().c_str());
		*/
		// Load table columns
		if (connect->SQL_OK(connect->LoadColumnsAttr(tablename))) {
			nbcols = connect->GetNbColumns(tablename);
			printf("\tColumns attributes loaded, column quantity : %d\n", nbcols);
			for (k = 0; k < nbcols; k++) {
				columnname = connect->GetColumnName(tablename, k);
				printf("\t\tColumn %d : %s, %s, size:%d, len:%d, dec:%d, prec:%d, null:%d, default:%s, ord:%d\n",
						k, columnname.c_str(),
						connect->GetColumnTypeName(tablename, columnname).c_str(),
						connect->GetColumnSize(tablename, columnname),
						connect->GetColumnBufLength(tablename, columnname),
						connect->GetColumnDecDigits(tablename, columnname),
						connect->GetColumnRadix(tablename, columnname),
						connect->GetColumnNullable(tablename, columnname),
						connect->GetColumnDefaultValue(tablename, columnname).c_str(),
						connect->GetColumnOrdinalPos(tablename, columnname));
			}
		}
		else printf("Error n %d : %s\n", connect->GetLastNumError(), connect->GetLastError().c_str());
	}
}

void Disconnect(VDKODBCConnection *connect)
{
	// Disconnect from data source
	connect->Disconnect();
	printf("\nDisconnected\n");
	delete connect;
}

void ShowResult(VDKODBCQuery *query)
{
	VDKString sbuffer;
	
	for (int j = 1; j <= query->ColumnCount(); j++) {
		printf("name %s, type %s, size %d, dec %d, nullable %d\n",
			   query->GetColumnName(j).c_str(),
			   query->GetColumnTypeName(query->GetColumnType(j)).c_str(),
			   query->GetColumnSize(j),
			   query->GetColumnDecDigits(j),
			   query->GetColumnNullable(j));
	}
	printf("First name\t\tLast name\t\tBirth date\t\tHeight\t\tWeight\n");
	printf("----------\t\t---------\t\t----------\t\t------\t\t------\n");
/* This is one way which doesn't work if RowCount miss to send a right value
   This happens on MSDE for example, it seems to depends on ODBC driver
	for(int i = 1; i <= query->RowCount(); i++) {
		query->Fetch();
		printf("%s\t\t\t", query->GetFieldValue("first_name").c_str());
		printf("%s\t\t", query->GetFieldValue("last_name").c_str());
		printf("%s\t\t", query->GetFieldValue("birthdate").c_str());
		printf("%s\t\t", query->GetFieldValue("height").c_str());
		printf("%s\n", query->GetFieldValue("weight").c_str());
	}
*/
	while (!query->Eof()) {
		printf("%s\t\t\t", query->GetFieldValue("first_name").c_str());
		printf("%s\t\t", query->GetFieldValue("last_name").c_str());
		printf("%s\t\t", query->GetFieldValue("birthdate").c_str());
		printf("%s\t\t", query->GetFieldValue("height").c_str());
		printf("%s\n", query->GetFieldValue("weight").c_str());
	}

	printf("%d record(s) found\n\n", query->RowCount());
}

int main(int argc,char *argv[])
{
	VDKODBCConnection *connect;
	VDKODBCQuery *query;
	VDKString qry;
	int rslt;

	printf("Starting VDKODBC test program\n");
	printf("Looking for '%s' database\n\n", DATABASENAME);
	connect = new VDKODBCConnection();
	if (!connect) return -1;
	// Drivers
	GetODBCDrivers(connect);
	// Sources
	GetODBCDataSources(connect);
	// Connection
	if (!ConnectedToDataSource(connect)) {
		delete connect;
		return -2;
	}
	// Attributes
	ShowTablesAttr(connect);
	// Data
	query = new VDKODBCQuery("test_query", connect);
	if (!query) {
		Disconnect(connect);
		return -3;
	}
	printf("\nTesting reading mode\n\n");
	// ODBC driver problem with PostgreSQL date style
	//query->ExecDirect("%s", "set datestyle = 'European,SQL'");
	qry.Sprintf("%s\n\t%s\n\t%s\n\t%s\n",
				"select child.first_name, child.last_name, child.birthdate, child.height, child.weight ",
				"from family child inner join family anc ",
				"on child.father = anc.id where anc.first_name = 'Pierre-Louis' and child.id != 0 ",
				"order by birthdate");
	printf("Query = %s\n", qry.c_str());
	query->ExecDirect(qry);
	ShowResult(query);
	query->Close();
	
	qry.Sprintf("%s\n\t%s\n\t%s\n",
				"select first_name, last_name, birthdate, height, weight ",
				"from family where id != 0 ",
				"order by birthdate");
	printf("Query = %s\n", qry.c_str());
	query->ExecDirect(qry);
	ShowResult(query);
	query->Close();
	
	qry.Sprintf("%s\n\t%s\n",
				"select first_name, last_name, birthdate, height, weight ",
				"from family where first_name like 'P%'");
	printf("Query = %s\n", qry.c_str());
	query->ExecDirect(qry);
	ShowResult(query);
	query->Close();

	printf("Testing writing mode\n\n");
	query->ExecDirect("BEGIN TRANSACTION");
	printf("Transaction started\n");
	
	qry.Sprintf("%s\n\t%s\n",
				"insert into family values",
				"(15, 'Future', 'Children', '2004/01/01', 0, 0, 10, 11)");
	printf("Query = %s\n", qry.c_str());
	rslt = query->ExecDirect(qry);
	if (rslt == SQL_SUCCESS) printf("\tInsert executed\n");
	else printf("\tInsert missed\n");
	
	if (rslt == SQL_SUCCESS) {
		qry.Sprintf("%s\n",
				"delete from family where first_name = 'Future'");
		printf("Query = %s\n", qry.c_str());
		rslt = query->ExecDirect(qry);
		if (rslt == SQL_SUCCESS) printf("\tDelete executed\n");
		else printf("\tDelete missed\n");
	}
	if (rslt == SQL_SUCCESS) {
		qry.Sprintf("%s\n",
				"update family set first_name = 'New one' where last_name = 'Children'");
		printf("Query = %s\n", qry.c_str());
		rslt = query->ExecDirect(qry);
		if (rslt == SQL_SUCCESS) printf("\tUpdate executed\n");
		else printf("\tUpdate missed\n");
	}
	if (rslt == SQL_SUCCESS) {
		query->ExecDirect("COMMIT TRANSACTION");
		printf("Transaction committed\n");
	}
	else {
		query->ExecDirect("ROLLBACK TRANSACTION");
		printf("Transaction rollbacked\n");
	}
	delete query;
	Disconnect(connect);
	return 0;
}
